@@ -35,25 +35,42 @@ http://blockattack.net
 #include "puzzlehandler.hpp"
 #include "stageclearhandler.hpp"
 #include <sstream>
+#include <deque>
 
 using std::string;
 using std::stringstream;
 using std::cerr;
-using std::endl;
 using std::vector;
 
-
-static stringstream converter;
-
 //Function to convert numbers to string (2 diget)
 static string itoa2(int num) {
-	converter.str(std::string());
-	converter.clear();
-	if (num<10) {
-		converter << "0";
+	char buffer[20];
+	snprintf(buffer, sizeof(buffer), "%02i", num);
+	return buffer;
+}
+
+/**
+ * This function tells how long the game may pause the rise of the blocks.
+ * This decreases as the game progresses.
+ * @param stops In out variable. If larger than the max allowed
+ * @param ticks Ticks since the game was started
+ * @param gameStartedAt Tick the game was started at.
+ */
+static void capMaxStops(int& stops, unsigned int ticks, unsigned int gameStartedAt) {
+	if (ticks < gameStartedAt || stops == 0) {
+		//Game not started or not stops, do nothing
+		return;
+	}
+	unsigned int gameTime = gameStartedAt - ticks;
+
+	int maxSeconds = 10-( (gameTime/1000) % 30 );
+	if (maxSeconds < 3) {
+		maxSeconds = 1;
+	}
+	int cappedStops = maxSeconds * 1000;
+	if (stops > cappedStops) {
+		stops = cappedStops;
 	}
-	converter << num;
-	return converter.str();
 }
 
 stageButton stageButtonStatus;
@@ -217,8 +234,9 @@ bool BlockGame::GetIsWinner()  const {
 	return hasWonTheGame;
 }
 
-
 void BlockGame::NewGame(const BlockGameStartInfo& s) {
+	replayInfo.startInfo = s;
+	replayInfo.actions.clear();
 	this->recordStats = s.recordStats;
 	if (s.AI) {
 		recordStats = false;
@@ -243,10 +261,11 @@ void BlockGame::NewGame(const BlockGameStartInfo& s) {
 			puzzleMode = true;
 			Level = s.level;
 			MovesLeft = PuzzleNumberOfMovesAllowed(Level);
-			for (int i=0; i<6; i++)
+			for (int i=0; i<6; i++) {
 				for (int j=0; j<12; j++) {
 					board[i][j+1] = PuzzleGetBrick(Level,i,j);
 				}
+			}
 			baseSpeed = 100000;
 			speed = 100000;
 
@@ -257,7 +276,7 @@ void BlockGame::NewGame(const BlockGameStartInfo& s) {
 				}
 			}
 			for (int j=0; j<6; j++) {
-				board[j][0] = 5; 
+				board[j][0] = 5;
 			}
 		}
 		this->singlePuzzle = s.singlePuzzle;
@@ -319,10 +338,11 @@ void BlockGame::NewGame( unsigned int ticks) {
 	pushedPixelAt = gameStartedAt;
 	nextGarbageNumber = 10;
 	handicap=0;
-	for (int i=0; i<7; i++)
+	for (int i=0; i<7; i++) {
 		for (int j=0; j<30; j++) {
 			board[i][j] = -1;
 		}
+	}
 	for (int i=0; i<NUMBEROFCHAINS; i++) {
 		chainUsed[i]=false;
 		chainSize[i] = 0;
@@ -330,6 +350,11 @@ void BlockGame::NewGame( unsigned int ticks) {
 	lastAImove = ticks+3000;
 }   //NewGame
 
+void BlockGame::FinalizeBlockGameInfo() {
+	replayInfo.extra.name = name;
+	replayInfo.extra.score = score;
+	replayInfo.extra.seconds = gameEndedAfter;
+}
 
 //Prints "winner" and ends game
 void BlockGame::setPlayerWon() {
@@ -344,13 +369,14 @@ void BlockGame::setPlayerWon() {
 			Stats::getInstance()->addOne("totalWins");
 			if (vsAI) {
 				//We have defeated an AI
-				Stats::getInstance()->addOne("defeatedAI"+itoa(getAIlevel()));
+				Stats::getInstance()->addOne("defeatedAI"+std::to_string(getAIlevel()));
 			}
 		}
 		if (AI_Enabled && vsAI) {
 			//The AI have defeated a human player
-			Stats::getInstance()->addOne("defeatedByAI"+itoa(getAIlevel()));
+			Stats::getInstance()->addOne("defeatedByAI"+std::to_string(getAIlevel()));
 		}
+		FinalizeBlockGameInfo();
 	}
 	hasWonTheGame = true;
 }
@@ -367,37 +393,43 @@ void BlockGame::setDraw() {
 	if (recordStats) {
 		Stats::getInstance()->addOne("totalDraws");
 	}
+	FinalizeBlockGameInfo();
 }
 
 
 //Test if LineNr is an empty line, returns false otherwise.
 bool BlockGame::LineEmpty(int lineNr) const {
 	bool empty = true;
-	for (int i = 0; i <7; i++)
+	for (int i = 0; i <7; i++) {
 		if (board[i][lineNr] != -1) {
 			empty = false;
 		}
+	}
 	return empty;
 }
 
 //Test if the entire board is empty (used for Puzzles)
 bool BlockGame::BoardEmpty() const {
 	bool empty = true;
-	for (int i=0; i<6; i++)
-		for (int j=1; j<13; j++)
+	for (int i=0; i<6; i++) {
+		for (int j=1; j<13; j++) {
 			if (board[i][j] != -1) {
 				empty = false;
 			}
+		}
+	}
 	return empty;
 }
 
 //Anything that the user can't move? In that case Game Over cannot occur
 bool BlockGame::hasStaticContent() const {
-	for (int i=0; i<6; i++)
-		for (int j=1; j<13; j++)
+	for (int i=0; i<6; i++) {
+		for (int j=1; j<13; j++) {
 			if (board[i][j] >= 10000000) {  //Higher than this means combos (garbage is static, but the stack is static but nothing to do about it)
 				return true;    //They are static
 			}
+		}
+	}
 	return false;                   //Return false if no static object found
 }
 
@@ -528,7 +560,7 @@ void BlockGame::putStartBlocks(int n) {
 void BlockGame::ReduceStuff() {
 	int howMuchHang = (ticks - FRAMELENGTH*hangTicks)/FRAMELENGTH;
 	if (howMuchHang>0) {
-		for (int i=0; i<7; i++)
+		for (int i=0; i<7; i++) {
 			for (int j=0; j<30; j++) {
 				if ((board[i][j]/BLOCKHANG)%10==1) {
 					int hangNumber = (board[i][j]/10)%100;
@@ -551,6 +583,7 @@ void BlockGame::ReduceStuff() {
 					}
 				}
 			}
+		}
 	}
 	hangTicks+=howMuchHang;
 }
@@ -595,7 +628,6 @@ bool BlockGame::CreateGarbage(int wide, int height) {
 		if (nextGarbageNumber>999999) {
 			nextGarbageNumber = 10;
 		}
-		//bGarbageFallLeft = !(bGarbageFallLeft);
 		return true;
 	}
 	return false;
@@ -717,19 +749,21 @@ void BlockGame::ClearBlocks() {
 					combo++;
 				}
 				else {
-					if (combo>2)
+					if (combo>2) {
 						for (int k = i-combo; k<i; k++) {
 							toBeCleared[j][k] = true;
 						}
+					}
 					combo=1;
 					previus = board[j][i]%10000000;
 				}
 			} //if board
 			else {
-				if (combo>2)
+				if (combo>2) {
 					for (int k = i-combo; k<i; k++) {
 						toBeCleared[j][k] = true;
 					}
+				}
 				combo = 0;
 				previus = -1;
 			}
@@ -738,11 +772,11 @@ void BlockGame::ClearBlocks() {
 	} //for j
 
 	chain = 0;
-	for (int i=0; i<6; i++)
+	for (int i=0; i<6; i++) {
 		for (int j=0; j<30; j++) {
 			//Clears blocks marked for clearing
 			int temp=board[i][j];
-			if (1==((temp/BLOCKWAIT)%10))
+			if (1==((temp/BLOCKWAIT)%10)) {
 				if (((temp/10)%100)==0) {
 					if (chainSize[chain]<chainSize[board[i][j]/10000000]) {
 						chain = board[i][j]/10000000;
@@ -753,7 +787,9 @@ void BlockGame::ClearBlocks() {
 					AddExplosion(i, j);
 					board[i][j]=-2;
 				}
+			}
 		}
+	}
 	for (int i=0; i<7; i++) {
 		bool setChain=false;
 		for (int j=0; j<30; j++) {
@@ -765,12 +801,11 @@ void BlockGame::ClearBlocks() {
 				setChain=true;
 				BlockPopEvent();
 			}
-			if (board[i][j]!=-1)
+			if (board[i][j]!=-1) {
 				if ((setChain)&&((board[i][j]/GARBAGE)%10!=1)&&((board[i][j]/GARBAGE)%10!=2)) {
 					board[i][j]=((board[i][j]%CHAINPLACE)+CHAINPLACE*chain);
-					//somethingsGottaFall = true;
 				}
-
+			}
 		}
 	}
 	int startvalue;
@@ -856,7 +891,6 @@ void BlockGame::ClearBlocks() {
 	chainSize[chain]++;
 	for (int i=0; i<30; i++) {
 		for (int j=0; j<6; j++) {
-			//if(board[j][i]/10==(BLOCKWAIT+10*FALLTIME)/10)
 			if (toBeCleared[j][i]) {
 				board[j][i]=(board[j][i]%10000000)+chain*10000000;
 			}
@@ -871,7 +905,7 @@ void BlockGame::ClearBlocks() {
 				if (toBeCleared[j][i]) {
 					if (!dead) {
 						dead=true;
-						string tempS = itoa(chainSize[chain]);
+						string tempS = std::to_string(chainSize[chain]);
 						if (chainSize[chain]>1) {
 							AddText(j, i, tempS, 1000);
 						}
@@ -994,7 +1028,7 @@ void BlockGame::ClearBlocks() {
 					LongChainDoneEvent();
 				}
 				if (chainSize[i]>1 && !puzzleMode && recordStats) {
-					Stats::getInstance()->addOne((string)"chainX"+itoa(chainSize[i]));
+					Stats::getInstance()->addOne((string)"chainX"+std::to_string(chainSize[i]));
 				}
 				chainUsed[i]=false;
 			}
@@ -1009,6 +1043,7 @@ void BlockGame::SetGameOver() {
 		if (recordStats) {
 			TimeHandler::addTime("playTime",TimeHandler::ms2ct(gameEndedAfter));
 		}
+		FinalizeBlockGameInfo();
 	}
 	bGameOver = true;
 	if (stageClear) {
@@ -1041,7 +1076,7 @@ int BlockGame::FallBlock(int x, int y, int number) {
 		i++;
 	}
 	if (canFall) {
-		//cout << "Now falling" << endl;
+		//cout << "Now falling" << "\n";
 		for (int j = x; j<i; j++) {
 			board[j][y-1] = board[j][y];
 			board[j][y] = -1;
@@ -1231,7 +1266,7 @@ void BlockGame::FindTowerHeight() {
 ///////////////////////////////////////////////////////////////////////////
 //First the helpet functions:
 int BlockGame::nrOfType(int line, int type) {
-	// cout << "Start_ nrOfType" << endl;
+	// cout << "Start_ nrOfType" << "\n";
 	int counter = 0;
 	for (int i=0; i<6; i++)
 		if (board[i][line]==type) {
@@ -1243,7 +1278,7 @@ int BlockGame::nrOfType(int line, int type) {
 
 //See if a combo can be made in this line
 int BlockGame::horiInLine(int line) {
-	//cout << "Start_ hori in line" << endl;
+	//cout << "Start_ hori in line" << "\n";
 	int nrOfType[7] = {0, 0, 0, 0, 0, 0, 0};
 	int iTemp;
 	int max = 0;
@@ -1263,7 +1298,7 @@ int BlockGame::horiInLine(int line) {
 }
 
 bool BlockGame::horiClearPossible() {
-	//cout << "Start_ horiclear possible" << endl;
+	//cout << "Start_ horiclear possible" << "\n";
 	int i=13;
 	bool solutionFound = false;
 	do {
@@ -1290,7 +1325,7 @@ bool BlockGame::lineHasGarbage(int line) {
 
 //Types 0..6 in line
 int BlockGame::nrOfRealTypes(int line) {
-	//cout << "Start_ nrOfReal" << endl;
+	//cout << "Start_ nrOfReal" << "\n";
 	int counter = 0;
 	for (int i=0; i<6; i++) {
 		if ((board[i][line]>-1)&&(board[i][line]<7)) {
@@ -1302,7 +1337,7 @@ int BlockGame::nrOfRealTypes(int line) {
 
 //See if there is a tower
 bool BlockGame::ThereIsATower() {
-	//cout << "Start_ there is a tower" << endl;
+	//cout << "Start_ there is a tower" << "\n";
 	bool bThereIsATower = false; //Unless proven otherwise!
 	bool topReached = false; //If we have reached the top
 	int lineNumber = 0;
@@ -1332,13 +1367,13 @@ bool BlockGame::ThereIsATower() {
 	}
 	while ((!bThereIsATower)&&(!topReached));
 	//if(bThereIsATower)
-	//cout << "There is actually a tower" << endl;
+	//cout << "There is actually a tower" << "\n";
 	return bThereIsATower;
 }
 
 double BlockGame::firstInLine1(int line) {
 	if (line > 20 || line < 0) {
-		cerr << "Warning: first in Line1: " << line << endl;
+		cerr << "Warning: first in Line1: " << line << "\n";
 		return 3.0;
 	}
 	for (int i=0; i<6; i++) {
@@ -1352,13 +1387,14 @@ double BlockGame::firstInLine1(int line) {
 //returns the first coordinate of the block of type
 double BlockGame::firstInLine(int line, int type) {
 	if (line > 20 || line < 0) {
-		cerr << "Warning: first in Line: " << line << endl;
+		cerr << "Warning: first in Line: " << line << "\n";
 		return 3.0;
 	}
-	for (int i=0; i<6; i++)
+	for (int i=0; i<6; i++) {
 		if (board[i][line]==type) {
 			return (double)i;
 		}
+	}
 	return 3.0;
 }
 
@@ -1378,7 +1414,7 @@ int BlockGame::closestTo(int line, int place) {
 
 //The AI will remove a tower
 void BlockGame::AI_ClearTower() {
-	//   cout << "AI: ClearTower, line: " << AIlineToClear << endl;
+	//   cout << "AI: ClearTower, line: " << AIlineToClear << "\n";
 	int place = (int)firstInLine(AIlineToClear-1, -1); //Find an empty field to frop a brick into
 	int xplace = closestTo(AIlineToClear, place); //Find the brick to drop in it
 	if (cursory+1<AIlineToClear) {
@@ -1440,11 +1476,12 @@ void BlockGame::AI_ClearHori() {
 		if (left<right) {
 			//   cout << ", right>left";
 			int count=0;
-			for (int i=0; (i<4)&&(count<1); i++)
+			for (int i=0; (i<4)&&(count<1); i++) {
 				if ((board[i][lowestLine]==AIcolorToClear)&&((i==0)||(board[i+1][lowestLine]!=AIcolorToClear))) {
 					count++;
 					xplace = i;
 				}
+			}
 		}
 		else {
 			//   cout << ", left>=right";
@@ -1473,7 +1510,7 @@ void BlockGame::AI_ClearHori() {
 	else {
 		AIstatus = 0;
 	}
-	//cout << endl; //for debugging
+	//cout << "\n"; //for debugging
 }
 
 //Test if vertical clear is possible
@@ -1490,7 +1527,6 @@ bool BlockGame::veriClearPossible() {
 				AIlineToClear = i;
 				found=true;
 			}
-
 		}
 	}
 	return found;
@@ -1519,14 +1555,14 @@ void BlockGame::AI_ClearVertical() {
 	int placeToCenter = (int)(firstInLine(AIlineToClear, AIcolorToClear)/3.0+firstInLine(AIlineToClear+1, AIcolorToClear)/3.0+firstInLine(AIlineToClear+2, AIcolorToClear)/3.0);
 	int unlimitedLoop=0;
 	if (AIlineToClear < 0 || AIlineToClear > 20) {
-		cerr << "AIlineToClear out of range: " << AIlineToClear << endl;
+		cerr << "AIlineToClear out of range: " << AIlineToClear << "\n";
 		return;
 	}
 	if (placeToCenter<0 || placeToCenter > 5) {
-		cerr << "placeToCenter out of range: " << placeToCenter << endl;
+		cerr << "placeToCenter out of range: " << placeToCenter << "\n";
 		return;
 	}
-	//cout << "AI_ClearVertical: " << placeToCenter << ", " << AIlineToClear << endl;
+	//cout << "AI_ClearVertical: " << placeToCenter << ", " << AIlineToClear << "\n";
 	while (((board[placeToCenter][AIlineToClear]>1000000)||(board[placeToCenter][AIlineToClear+1]>1000000)||(board[placeToCenter][AIlineToClear+2]>1000000))&&(unlimitedLoop<10)) {
 		unlimitedLoop++;
 		placeToCenter++;
@@ -1617,7 +1653,7 @@ void BlockGame::AI_ClearVertical() {
 	if ((nrOfType(AIlineToClear, AIcolorToClear)==0)||(nrOfType(AIlineToClear+1, AIcolorToClear)==0)||(nrOfType(AIlineToClear+2, AIcolorToClear)==0)) {
 		AIstatus = 0;
 	}
-	//cout << endl;
+	//cout << "\n";
 }
 
 
@@ -1682,7 +1718,7 @@ void BlockGame::AI_Move() {
 //Updates evrything, if not called nothing happends
 void BlockGame::Update() {
 	unsigned int nowTime = ticks; //We remember the time, so it doesn't change during this call
-
+	capMaxStops(stop, nowTime, gameStartedAt);
 	{
 		FindTowerHeight();
 		if ((linesCleared-TowerHeight>stageClearLimit) && (stageClear) && (!bGameOver)) {
@@ -1780,14 +1816,63 @@ bool BlockGame::IsNearDeath() const {
 }
 
 void BlockGame::UpdateInternal(unsigned int newtick) {
-	while (newtick >= ticks+50) {
-		ticks+=50;
+	while (newtick >= ticks+10) {
+		ticks+=10;
 		Update();
 	}
 }
 
-void BlockGame::Update(unsigned int newtick) {
-	UpdateInternal(newtick);
+void BlockGame::DoAction (const BlockGameAction& action) {
+	if (action.action == BlockGameAction::Action::UPDATE && action.tick < ticks+10) {
+		return;  //Ignore if this is an update and not high enough
+	}
+	if (action.action == BlockGameAction::Action::UPDATE && replayInfo.actions.size() > 0 && replayInfo.actions.back().action == action.action) {
+		replayInfo.actions.back() = action;
+	}
+	else {
+		replayInfo.actions.push_back(action);
+	}
+	if (action.action == BlockGameAction::Action::UPDATE) {
+		UpdateInternal(action.tick);
+	}
+	if (action.action == BlockGameAction::Action::SET_DRAW) {
+		setDraw();
+	}
+	if (action.action == BlockGameAction::Action::SET_WON) {
+		setPlayerWon();
+	}
+	if (action.action == BlockGameAction::Action::SET_GAME_OVER) {
+		SetGameOver();
+	}
+	if (action.action == BlockGameAction::Action::PUSH) {
+		PushLine();
+	}
+	if (action.action == BlockGameAction::Action::MOVE) {
+		char direction = action.way;
+		MoveCursor(direction);
+	}
+	if (action.action == BlockGameAction::Action::SWITCH) {
+		SwitchAtCursor();
+	}
+	if (action.action == BlockGameAction::Action::PUSH_GARBAGE) {
+		for (const GarbageStruct& i : action.garbage) {
+			if (i.greyGarbage) {
+				CreateGreyGarbage();
+			}
+			else {
+				CreateGarbage(i.width, i.height);
+			}
+		}
+	}
+	if (action.action == BlockGameAction::Action::MOUSE_DOWN) {
+		MouseDown(action.x, action.y);
+	}
+	if (action.action == BlockGameAction::Action::MOUSE_UP) {
+		MouseUp();
+	}
+	if (action.action == BlockGameAction::Action::MOUSE_MOVE) {
+		MouseMove(action.x);
+	}
 }
 
 bool BlockGame::isSinglePuzzle() const {
@@ -1805,7 +1890,7 @@ void BlockGame::PopSendGarbage(std::vector<GarbageStruct>& poppedData) {
 	this->garbageSendQueue.clear();
 }
 
-void BlockGame::GetMouseCursor(bool& pressed, int& x, int&y) const {
+void BlockGame::GetMouseCursor(bool& pressed, int& x, int& y) const {
 	if (mouse_cursorx < 0 || mouse_cursory < 0 || mouse_cursorx >=6 || mouse_cursory > 13) {
 		pressed = false;
 		x = 0;
@@ -1824,7 +1909,7 @@ void BlockGame::MouseDown(int x, int y) {
 	}
 	mouse_cursorx = x;
 	mouse_cursory = y;
-} 
+}
 
 void BlockGame::MouseMove(int x) {
 	if (AI_Enabled) {
