@@ -23,19 +23,18 @@ http://blockattack.net
 #include "global.hpp"
 #include <iostream>
 #include <stdlib.h>
-#include "menu/MenuItem.hpp"
 #include "MenuSystem.h"
 #include "common.h"
 
 using std::string;
 using std::cerr;
-using std::endl;
+using std::cout;
 using std::vector;
 
 #if 0
 //Menu
 static void PrintHi(Button* b) {
-	cout << "Hi" <<endl;
+	cout << "Hi" <<"\n";
 }
 #endif
 
@@ -55,8 +54,8 @@ extern control keySettings[3];
 //Function to return the name of a key, to be displayed...
 static string getKeyName(SDL_Keycode key) {
 	string keyname(SDL_GetKeyName(key));
-	if (verboseLevel) {
-		cout << key << " translated to " << keyname << endl;
+	if (globalData.verboseLevel) {
+		cout << key << " translated to " << keyname << "\n";
 	}
 	return keyname;
 }
@@ -96,74 +95,81 @@ void Button_changekey::doAction() {
 }
 
 void InitMenues() {
-	standardButton.setSurfaces(&menuMarked,&menuUnmarked);
-	standardButton.thefont = &nf_scoreboard_font;
+	standardButton.setSurfaces();
+	standardButton.thefont = &globalData.nf_button_font;
 }
 
-
-
-static void runSinglePlayerEndless(Button* b) {
-	runGame(0,0);
+static void runSinglePlayerEndless() {
+	runGame(Gametype::SinglePlayerEndless, 0);
 }
 
-static void runSinglePlayerTimeTrial(Button* b) {
-	runGame(1,0);
+static void runSinglePlayerTimeTrial() {
+	runGame(Gametype::SinglePlayerTimeTrial, 0);
 }
 
-static void runStageClear(Button* b) {
-	runGame(2,0);
+static void runStageClear() {
+	runGame(Gametype::StageClear, 0);
 }
 
-static void runSinglePlayerPuzzle(Button* b) {
-	runGame(3,0);
+static void runSinglePlayerPuzzle() {
+	runGame(Gametype::Puzzle, 0);
 }
 
-static void runSinglePlayerVs(Button* b) {
-	runGame(4,b->iGeneric1);
+class RunSinglePlayerVsButton : public Button {
+	virtual void doAction() override {
+		runGame(Gametype::SinglePlayerVs, iGeneric1);
+	}
+};
+
+static void runTwoPlayerTimeTrial() {
+	runGame(Gametype::TwoPlayerTimeTrial, 0);
 }
 
-static void runTwoPlayerTimeTrial(Button* b) {
-	runGame(10,0);
+static void runTwoPlayerVs() {
+	runGame(Gametype::TwoPlayerVs, 0);
 }
 
-static void runTwoPlayerVs(Button* b) {
-	runGame(11,0);
-}
+class MusicButton : public Button {
+	virtual void doAction() override {
+		globalData.MusicEnabled = !globalData.MusicEnabled;
+		setLabel(globalData.MusicEnabled? _("Music: On") : _("Music: Off"));
+	}
 
-static void buttonActionMusic(Button* b) {
-	MusicEnabled = !MusicEnabled;
-	b->setLabel(MusicEnabled? _("Music: On") : _("Music: Off"));
-}
+};
 
-static void buttonActionSound(Button* b) {
-	SoundEnabled = !SoundEnabled;
-	b->setLabel(SoundEnabled? _("Sound: On") : _("Sound: Off") );
-}
+class SoundButton : public Button {
+	virtual void doAction() override {
+		globalData.SoundEnabled = !globalData.SoundEnabled;
+		setLabel(globalData.SoundEnabled? _("Sound: On") : _("Sound: Off") );
+	}
+};
 
-static void buttonActionFullscreen(Button* b) {
-	bFullscreen = !bFullscreen;
-	b->setLabel(bFullscreen? _("Fullscreen: On") : _("Fullscreen: Off") );
-	ResetFullscreen();
-}
+class FullscreenButton : public Button {
+	virtual void doAction() override {
+		globalData.bFullscreen = !globalData.bFullscreen;
+		setLabel(globalData.bFullscreen? _("Fullscreen: On") : _("Fullscreen: Off") );
+		ResetFullscreen();
+	}
+};
 
-static void buttonActionPlayer1Name(Button* b) {
-	if ( OpenDialogbox(200, 100, player1name, _("Enter player 1 name:")) ) {
+static void buttonActionPlayer1Name() {
+	if ( OpenDialogbox(200, 100, globalData.player1name, _("Enter player 1 name:")) ) {
 		return;    //must save if true
 	}
 }
 
-static void buttonActionPlayer2Name(Button* b) {
-	if ( OpenDialogbox(200, 100, player2name, _("Enter player 2 name:")) ) {
+static void buttonActionPlayer2Name() {
+	if ( OpenDialogbox(200, 100, globalData.player2name, _("Enter player 2 name:")) ) {
 		return;    //must save if true
 	}
 }
 
-static void buttonActionHighscores(Button* b) {
+static void buttonActionHighscores() {
 	OpenScoresDisplay();
 }
 
 static void ChangeKeysMenu(long playernumber) {
-	Menu km(screen,_("Change key bindings"),true);
+	Menu km(globalData.screen,_("Change key bindings"),true);
 	Button_changekey bLeft(&keySettings[playernumber].left,_("Left") );
 	Button_changekey bRight(&keySettings[playernumber].right,_("Right") );
 	Button_changekey bUp(&keySettings[playernumber].up,_("Up") );
@@ -176,27 +182,27 @@ static void ChangeKeysMenu(long playernumber) {
 	km.addButton(&bDown);
 	km.addButton(&bPush);
 	km.addButton(&bSwitch);
-	km.run();
+	RunGameState(km);
 }
 
-static void ChangeKeysMenu1(Button* b) {
+static void ChangeKeysMenu1() {
 	ChangeKeysMenu(0);
 }
 
-static void ChangeKeysMenu2(Button* b) {
+static void ChangeKeysMenu2() {
 	ChangeKeysMenu(2);
 }
 
-static void ConfigureMenu(Button* b) {
-	Menu cm(screen,_("Configuration"),true);
-	Button bMusic,bSound,buttonFullscreen,bPlayer1Name,bPlayer2Name;
+static void ConfigureMenu() {
+	Menu cm(globalData.screen,_("Configuration"),true);
+	Button bPlayer1Name,bPlayer2Name;
 	Button bPlayer1Keys, bPlayer2Keys;
-	bMusic.setLabel(MusicEnabled? _("Music: On") : _("Music: Off") );
-	bMusic.setAction(buttonActionMusic);
-	bSound.setLabel(SoundEnabled? _("Sound: On") : _("Sound: Off") );
-	bSound.setAction(buttonActionSound);
-	buttonFullscreen.setLabel(bFullscreen? _("Fullscreen: On") : _("Fullscreen: Off") );
-	buttonFullscreen.setAction(buttonActionFullscreen);
+	MusicButton bMusic;
+	SoundButton bSound;
+	FullscreenButton buttonFullscreen;
+	bMusic.setLabel(globalData.MusicEnabled? _("Music: On") : _("Music: Off") );
+	bSound.setLabel(globalData.SoundEnabled? _("Sound: On") : _("Sound: Off") );
+	buttonFullscreen.setLabel(globalData.bFullscreen? _("Fullscreen: On") : _("Fullscreen: Off") );
 	bPlayer1Name.setAction(buttonActionPlayer1Name);
 	bPlayer1Name.setLabel(_("Change player 1's name") );
 	bPlayer2Name.setAction(buttonActionPlayer2Name);
@@ -212,19 +218,12 @@ static void ConfigureMenu(Button* b) {
 	cm.addButton(&bPlayer2Name);
 	cm.addButton(&bPlayer1Keys);
 	cm.addButton(&bPlayer2Keys);
-	cm.run();
+	RunGameState(cm);
 }
 
-static void SinglePlayerVsMenu(Button* b) {
-	Menu spvs(screen,_("Single player VS"),true);
-	Button d1,d2,d3,d4,d5,d6,d7;
-	d1.setAction(runSinglePlayerVs);
-	d2.setAction(runSinglePlayerVs);
-	d3.setAction(runSinglePlayerVs);
-	d4.setAction(runSinglePlayerVs);
-	d5.setAction(runSinglePlayerVs);
-	d6.setAction(runSinglePlayerVs);
-	d7.setAction(runSinglePlayerVs);
+static void SinglePlayerVsMenu() {
+	Menu spvs(globalData.screen,_("Single player VS"),true);
+	RunSinglePlayerVsButton d1,d2,d3,d4,d5,d6,d7;
 	d1.setPopOnRun(true);
 	d2.setPopOnRun(true);
 	d3.setPopOnRun(true);
@@ -253,24 +252,24 @@ static void SinglePlayerVsMenu(Button* b) {
 	spvs.addButton(&d5);
 	spvs.addButton(&d6);
 	spvs.addButton(&d7);
-	spvs.run();
+	RunGameState(spvs);
 }
 
-static void MultiplayerMenu(Button* b) {
-	Menu mm(screen,_("Multiplayer"),true);
-	Button bTT, bVs, bNet;
+static void MultiplayerMenu() {
+	Menu mm(globalData.screen,_("Multiplayer"),true);
+	Button bTT, bVs;
 	bTT.setLabel(_("Two player - time trial"));
 	bTT.setAction(runTwoPlayerTimeTrial);
 	bVs.setLabel(_("Two player - vs"));
 	bVs.setAction(runTwoPlayerVs);
 	mm.addButton(&bTT);
 	mm.addButton(&bVs);
-	mm.run();
+	RunGameState(mm);
 }
 
 void MainMenu() {
 	InitMenues();
-	Menu m(screen,_("Block Attack - Rise of the blocks"),false);
+	Menu m(globalData.screen,_("Block Attack - Rise of the blocks"),false);
 	Button bHi,bTimetrial1, bStageClear, bPuzzle, bVs1, bMulti, bConfigure,bHighscore;
 	bHi.setLabel(_("Single player - endless") );
 	bHi.setAction(runSinglePlayerEndless);
@@ -296,5 +295,5 @@ void MainMenu() {
 	m.addButton(&bMulti);
 	m.addButton(&bConfigure);
 	m.addButton(&bHighscore);
-	m.run();
+	RunGameState(m);
 }
