@@ -6,7 +6,7 @@ test $1
 POFILE=$1
 
 LANGUAGE=${1##*/}
-LANGUAGE=${LANGUAGE%%_*}
+LANGUAGE=${LANGUAGE%%.*}
 mkdir -p locale/$LANGUAGE/LC_MESSAGES
 echo "Building $LANGUAGE from $1"
 msgfmt -o locale/$LANGUAGE/LC_MESSAGES/blockattack_roftb.mo $1
